import { logger } from './salesforce.js';
import util from 'util';
import { exceptionalError } from './salesforce.js';
export function queryGenerator(targetType, filters, overriddenTargetFields = null) {
    if (!(targetType.createNew)) {
        throw `Invalid targetType provded to queryGenerator`;
    }
    if (!filters) {
        filters = {};
    }
    let selectString = getSelectionSubstring(targetType, overriddenTargetFields);
    let targetObject = targetType.objConstants.apiName;
    let whereString = getWhereSubstring(targetType, filters);
    return `q=SELECT ${selectString} FROM ${targetObject}${whereString}`;
}
function getSelectionSubstring(targetType, overriddenTargetFields) {
    let selectFields = [];
    if (!overriddenTargetFields && (!targetType.objConstants.targetFields || targetType.objConstants.targetFields.length === 0)) {
        selectFields = Object.entries(targetType.objConstants.apiMapping).map(([k, v]) => v);
    }
    else {
        let targetFields = null;
        if (overriddenTargetFields && Array.isArray(overriddenTargetFields) && overriddenTargetFields.length > 0) {
            targetFields = overriddenTargetFields;
        }
        else {
            targetFields = targetType.objConstants.targetFields;
        }
        if (Array.isArray(targetFields)) {
            for (const reqField of targetFields) {
                if (!targetType.objConstants.apiMapping.hasOwnProperty(reqField)) {
                    throw new exceptionalError(`Unsupported query type ${reqField} for ${targetType.objConstants.apiName}`);
                }
                //@ts-ignore got no time for that
                selectFields.push(`${targetType.objConstants.apiMapping[reqField]}`);
            }
        }
        else {
            logger.info(`${util.inspect(targetFields, { depth: null })}`);
            throw new exceptionalError(`Called querygenerator somehow with an invalid selector list??`);
        }
    }
    return selectFields.join(", ");
}
function getWhereSubstring(targetType, filters) {
    let queryFields = [];
    if (Object.keys(filters).length > 0) {
        for (const [niceName, selectVal] of Object.entries(filters)) {
            if (selectVal === null || selectVal === '') {
                continue;
            }
            if (!targetType.objConstants.apiMapping.hasOwnProperty(niceName)) {
                throw new exceptionalError(`Unsupported filter ${niceName} for query type ${targetType.objConstants.apiName}`);
            }
            if (Array.isArray(selectVal)) {
                //@ts-ignore cant be arsed
                let orStr = `${selectVal.filter((a) => (a != null)).map((a) => (`${targetType.objConstants.apiMapping[niceName]} ${formatSelectionComparitor(a)}`)).join(" OR ")}`;
                if (selectVal.length > 1) {
                    orStr = `(${orStr})`;
                }
                queryFields.push(orStr);
            }
            else {
                if (selectVal !== null) {
                    //@ts-ignore some day I'll do this right
                    queryFields.push(`${targetType.objConstants.apiMapping[niceName]} ${formatSelectionComparitor(selectVal)}`);
                }
            }
        }
    }
    let whereString = ``;
    if (queryFields.length > 0) {
        whereString = ` WHERE ${queryFields.join(" AND ")}`;
    }
    return whereString;
}
function formatSelectionComparitor(stringVal) {
    const comparisonRegex = /(>|<|>=|<=|=) ?(\d+?|\d{4}-\d{2}-\d{2})$/;
    const likeRegex = /^like .+?'$/i;
    if (stringVal === true || stringVal === false) {
        return `= ${stringVal}`;
    }
    else {
        if (typeof (stringVal) === 'string') {
            let regexCap = stringVal.match(comparisonRegex);
            if (regexCap) {
                if (!regexCap[1] || !regexCap[2]) {
                    logger.info(`Something went wrong trying to interpret the selector string val ${stringVal}`);
                    throw new exceptionalError(`Bad string val for regex`);
                }
                let comparitor = regexCap[1];
                let compareTo = regexCap[2];
                if (compareTo.match(/\d{4}-\d{2}-\d{2}/)) {
                    compareTo = `${compareTo}T00:00:00Z`;
                }
                return `${comparitor} ${compareTo}`;
            }
            else {
                if (stringVal.match(likeRegex)) {
                    return stringVal;
                }
            }
        }
    }
    return `= '${stringVal}'`;
}
export function compareStreets(newStreet, existingStreet) {
    let nowTime = Number((new Date()));
    let toMatch = `${existingStreet.toUpperCase()}`;
    let newCandidate = `${newStreet.toUpperCase()}`;
    for (const [regexVal, replaceWith] of Object.entries(fixThese)) {
        let regExp = new RegExp(`(?<=\\W|^)${regexVal}(?=\\W|$)`, "g");
        toMatch = toMatch.replace(regExp, replaceWith);
        newCandidate = newCandidate.replace(regExp, replaceWith);
    }
    let oldNumMatch = toMatch.match(/^\w+?\b/i);
    if (oldNumMatch) {
        let oldNum = oldNumMatch[0];
        let newRegexp = new RegExp(`(^[^0-9]+?,? ?)${oldNum}`, "i");
        let newMatch = newCandidate.match(newRegexp);
        if (newMatch) {
            let replaceRegex = new RegExp(`${newMatch[1]}`, "i");
            newCandidate = newCandidate.replace(replaceRegex, '');
        }
    }
    toMatch = toMatch.replace(/NORTH/g, "N").replace(/SOUTH/g, "S").replace(/EAST/g, "E").replace(/WEST/g, "W");
    newCandidate = newCandidate.replace(/NORTH/g, "N").replace(/SOUTH/g, "S").replace(/EAST/g, "E").replace(/WEST/g, "W");
    newCandidate = newCandidate.replace(/(?:att|attention|attn|atn) ?:? ?(?:\w+?) (?:\w+?)\b/i, '');
    toMatch = toMatch.replace(/\W/g, '').match(/./g).sort().join("");
    newCandidate = newCandidate.replace(/\W/g, '').match(/./g).sort().join("");
    return (toMatch === newCandidate);
}
export function setDupeFiltersForObj(locationObj) {
    let duplicateSelectionFilter = {};
    if (!validateDupeFilters(locationObj)) {
        return null;
    }
    duplicateSelectionFilter.city = `${locationObj.details.address.city}`;
    duplicateSelectionFilter.province = `${locationObj.details.address.province}`;
    duplicateSelectionFilter.country = `${locationObj.details.address.country}`;
    duplicateSelectionFilter.postalCode = `${locationObj.details.address.postalCode}`;
    duplicateSelectionFilter.streetAddress = `${locationObj.details.address.streetAddress}`;
    duplicateSelectionFilter.accountId = locationObj.details.accountId;
    return setDupeFilters(duplicateSelectionFilter);
}
export function setDupeFilters(duplicateSelectionFilter) {
    if (duplicateSelectionFilter.city) {
        duplicateSelectionFilter.city = `LIKE '%${duplicateSelectionFilter.city.replace(/\W/g, '%')}%'`;
    }
    if (duplicateSelectionFilter.postalCode && duplicateSelectionFilter.country) {
        duplicateSelectionFilter.postalCode = setDupeLookupPostalCode(duplicateSelectionFilter.country, duplicateSelectionFilter.postalCode);
    }
    // todo - some day do this right!
    if (duplicateSelectionFilter.province) {
        duplicateSelectionFilter.province = `LIKE '${duplicateSelectionFilter.province}%'`;
    }
    if (duplicateSelectionFilter.streetAddress) {
        let streetNumCap = duplicateSelectionFilter.streetAddress.match(/^.+?(?=[a-z])/i);
        let streetNumString = "";
        if (streetNumCap) {
            streetNumString = streetNumCap[0].replace(/ $/i, '');
            streetNumString = streetNumString.replace(/^\D+?(?=\d)/i, '%');
        }
        else {
            logger.info(`Failed to get match - ${duplicateSelectionFilter.streetAddress} gave ${util.inspect(streetNumCap, { depth: null })}`);
            return null;
        }
        duplicateSelectionFilter.streetAddress = `LIKE '${streetNumString}%'`;
    }
    return duplicateSelectionFilter;
}
function validateDupeFilters(locationObj) {
    if (!locationObj.details.accountId) {
        return false;
    }
    const reqAddressFields = [
        'streetAddress',
        'city',
        'province',
        'postalCode',
        'country'
    ];
    for (const reqField of reqAddressFields) {
        //@ts-ignore
        if (!locationObj.details.address[reqField]) {
            //@ts-ignore
            logger.info(`Field ${reqField} was ${locationObj.details.address[reqField]}`);
            return false;
        }
    }
    return true;
}
function setDupeLookupPostalCode(country, postalCode) {
    if (country.toLocaleLowerCase() === 'us') {
        return `LIKE '%${postalCode.replace(/[ -]\d{4}$/, '')}%'`;
    }
    else if (country.toLocaleLowerCase() === 'ca') {
        let postalCodeNoSpaces = postalCode.replace(/ /, '');
        return `LIKE '${postalCodeNoSpaces.substring(0, 3)}%${postalCodeNoSpaces.substring(3, 6)}'`;
    }
    else {
        throw `Unsupported country ${country}`;
    }
}
export function validateCash(orderObj, orderDeets) {
    let moneyField = orderObj.details.money;
    let errMsgs = [];
    const moneyFieldNames = ['moneyManualAdjustments', 'moneyShipping', 'moneyTax', 'moneyUnitCost', 'moneyTotal'];
    if (orderDeets.type === "new" && orderDeets.source === "default") {
        if (!moneyField.moneyTotal) {
            errMsgs.push(`New orders must have a grand total`);
        }
        if (!moneyField.moneyCurrency) {
            errMsgs.push(`Transaction currency required`);
        }
        if (errMsgs.length > 0) {
            throw new exceptionalError(errMsgs.join("\n"));
        }
        for (const fieldName of moneyFieldNames) {
            //@ts-ignore
            if (!moneyField[fieldName]) {
                //@ts-ignore
                moneyField[fieldName] = 0;
            }
        }
        const calculatedTotal = moneyField.moneyUnitCost * orderObj.details.fulfillment.qtyRequested + moneyField.moneyShipping + moneyField.moneyTax + moneyField.moneyManualAdjustments;
        if (calculatedTotal !== moneyField.moneyTotal) {
            throw new exceptionalError(`Grand total was $${moneyField.moneyTotal} but should have been $${calculatedTotal}`);
        }
    }
    else {
        for (const fieldName of moneyFieldNames) {
            //@ts-ignore
            if (moneyField[fieldName] && moneyField[fieldName] !== 0) {
                //@ts-ignore
                orderDeets.notes.push(`Overrode ${fieldName} from ${moneyField[fieldName]} to 0`);
                //@ts-ignore
                moneyField[fieldName] = 0;
            }
        }
    }
}
const fixThese = {
    "ALLEE": "ALY",
    "ALLEY": "ALY",
    "ALLY": "ALY",
    "ANEX": "ANX",
    "ANNEX": "ANX",
    "ANNX": "ANX",
    "AV": "AVE",
    "AVEN": "AVE",
    "AVENU": "AVE",
    "AVENUE": "AVE",
    "AVN": "AVE",
    "AVNUE": "AVE",
    "BEACH": "BCH",
    "BEND": "BND",
    "BLUF": "BLF",
    "BLUFF": "BLF",
    "BLUFFS": "BLFS",
    "BOT": "BTM",
    "BOTTM": "BTM",
    "BOTTOM": "BTM",
    "BOUL": "BLVD",
    "BOULEVARD": "BLVD",
    "BOULV": "BLVD",
    "BRANCH": "BR",
    "BRDGE": "BRG",
    "BRIDGE": "BRG",
    "BRNCH": "BR",
    "BROOK": "BRK",
    "BROOKS": "BRKS",
    "BURG": "BG",
    "BURGS": "BGS",
    "BYPA": "BYP",
    "BYPAS": "BYP",
    "BYPASS": "BYP",
    "BYPS": "BYP",
    "CAMP": "CP",
    "CANYN": "CYN",
    "CANYON": "CYN",
    "CAPE": "CPE",
    "CAUSEWAY": "CSWY",
    "CAUSWA": "CSWY",
    "CEN": "CTR",
    "CENT": "CTR",
    "CENTER": "CTR",
    "CENTERS": "CTRS",
    "CENTR": "CTR",
    "CENTRE": "CTR",
    "CIRC": "CIR",
    "CIRCL": "CIR",
    "CIRCLE": "CIR",
    "CIRCLES": "CIRS",
    "CLIFF": "CLF",
    "CLIFFS": "CLFS",
    "CLUB": "CLB",
    "CMP": "CP",
    "CNTER": "CTR",
    "CNTR": "CTR",
    "CNYN": "CYN",
    "COMMON": "CMN",
    "COMMONS": "CMNS",
    "CORNER": "COR",
    "CORNERS": "CORS",
    "COURSE": "CRSE",
    "COURT": "CT",
    "COURTS": "CTS",
    "COVE": "CV",
    "COVES": "CVS",
    "CRCL": "CIR",
    "CRCLE": "CIR",
    "CREEK": "CRK",
    "CRESCENT": "CRES",
    "CREST": "CRST",
    "CROSSING": "XING",
    "CROSSROAD": "XRD",
    "CROSSROADS": "XRDS",
    "CRSENT": "CRES",
    "CRSNT": "CRES",
    "CRSSNG": "XING",
    "CURVE": "CURV",
    "DALE": "DL",
    "DAM": "DM",
    "DIV": "DV",
    "DIVIDE": "DV",
    "DRIV": "DR",
    "DRIVE": "DR",
    "DRIVES": "DRS",
    "DRV": "DR",
    "DVD": "DV",
    "ESTATE": "EST",
    "ESTATES": "ESTS",
    "EXP": "EXPY",
    "EXPR": "EXPY",
    "EXPRESS": "EXPY",
    "EXPRESSWAY": "EXPY",
    "EXPW": "EXPY",
    "EXTENSION": "EXT",
    "EXTN": "EXT",
    "EXTNSN": "EXT",
    "FALLS": "FLS",
    "FIELD": "FLD",
    "FIELDS": "FLDS",
    "FLAT": "FLT",
    "FORDS": "FRDS",
    "FOREST": "FRST",
    "FORESTS": "FRST",
    "FORG": "FRG",
    "FORGE": "FRG",
    "FORGES": "FRGS",
    "FORK": "FRK",
    "FORKS": "FRKS",
    "FORT": "FT",
    "FREEWAY": "FWY",
    "FREEWY": "FWY",
    "FRRY": "FRY",
    "FRT": "FT",
    "FRWAY": "FWY",
    "FRWY": "FWY",
    "GARDEN": "GDN",
    "GARDENS": "GDNS",
    "GARDN": "GDN",
    "GATEWAY": "GTWY",
    "GATEWY": "GTWY",
    "GATWAY": "GTWY",
    "GLEN": "GLN",
    "GLENS": "GLNS",
    "GRDEN": "GDN",
    "GRDN": "GDN",
    "GRDNS": "GDNS",
    "GREEN": "GRN",
    "GREENS": "GRNS",
    "GROV": "GRV",
    "GROVE": "GRV",
    "GROVES": "GRVS",
    "GTWAY": "GTWY",
    "HARB": "HBR",
    "HARBOR": "HBR",
    "HARBORS": "HBRS",
    "HARBR": "HBR",
    "HAVEN": "HVN",
    "HIGHWAY": "HWY",
    "HIGHWY": "HWY",
    "HILL": "HL",
    "HILLS": "HLS",
    "HIWAY": "HWY",
    "HIWY": "HWY",
    "HLLW": "HOLW",
    "HOLLOW": "HOLW",
    "HOLLOWS": "HOLW",
    "HOLWS": "HOLW",
    "HRBOR": "HBR",
    "HT": "HTS",
    "HWAY": "HWY",
    "ISLAND": "IS",
    "ISLANDS": "ISS",
    "ISLES": "ISLE",
    "ISLND": "IS",
    "ISLNDS": "ISS",
    "JCTION": "JCT",
    "JCTN": "JCT",
    "JCTNS": "JCTS",
    "JUNCTION": "JCT",
    "JUNCTIONS": "JCTS",
    "JUNCTN": "JCT",
    "JUNCTON": "JCT",
    "KEY": "KY",
    "KEYS": "KYS",
    "KNOL": "KNL",
    "KNOLL": "KNL",
    "KNOLLS": "KNLS",
    "LAKE": "LK",
    "LAKES": "LKS",
    "LANDING": "LNDG",
    "LANE": "LN",
    "LDGE": "LDG",
    "LIGHT": "LGT",
    "LIGHTS": "LGTS",
    "LNDNG": "LNDG",
    "LOAF": "LF",
    "LOCK": "LCK",
    "LOCKS": "LCKS",
    "LODG": "LDG",
    "LODGE": "LDG",
    "LOOPS": "LOOP",
    "MANOR": "MNR",
    "MANORS": "MNRS",
    "MDW": "MDWS",
    "MEADOW": "MDW",
    "MEADOWS": "MDWS",
    "MEDOWS": "MDWS",
    "MILL": "ML",
    "MILLS": "MLS",
    "MISSN": "MSN",
    "MNT": "MT",
    "MNTAIN": "MTN",
    "MNTN": "MTN",
    "MNTNS": "MTNS",
    "MOTORWAY": "MTWY",
    "MOUNT": "MT",
    "MOUNTAIN": "MTN",
    "MOUNTAINS": "MTNS",
    "MOUNTIN": "MTN",
    "MSSN": "MSN",
    "MTIN": "MTN",
    "NECK": "NCK",
    "ORCHARD": "ORCH",
    "ORCHRD": "ORCH",
    "OVERPASS": "OPAS",
    "OVL": "OVAL",
    "PARKS": "PARK",
    "PARKWAY": "PKWY",
    "PARKWAYS": "PKWY",
    "PARKWY": "PKWY",
    "PASSAGE": "PSGE",
    "PATHS": "PATH",
    "PIKES": "PIKE",
    "PINE": "PNE",
    "PINES": "PNES",
    "PKWAY": "PKWY",
    "PKWYS": "PKWY",
    "PKY": "PKWY",
    "PLAIN": "PLN",
    "PLAINS": "PLNS",
    "PLAZA": "PLZ",
    "PLZA": "PLZ",
    "POINT": "PT",
    "POINTS": "PTS",
    "PORT": "PRT",
    "PORTS": "PRTS",
    "PRAIRIE": "PR",
    "PRK": "PARK",
    "PRR": "PR",
    "RAD": "RADL",
    "RADIAL": "RADL",
    "RADIEL": "RADL",
    "RANCH": "RNCH",
    "RANCHES": "RNCH",
    "RAPID": "RPD",
    "RAPIDS": "RPDS",
    "RDGE": "RDG",
    "REST": "RST",
    "RIDGE": "RDG",
    "RIDGES": "RDGS",
    "RIVER": "RIV",
    "RIVR": "RIV",
    "RNCHS": "RNCH",
    "ROAD": "RD",
    "ROADS": "RDS",
    "ROUTE": "RTE",
    "RVR": "RIV",
    "SHOAL": "SHL",
    "SHOALS": "SHLS",
    "SHOAR": "SHR",
    "SHOARS": "SHRS",
    "SHORE": "SHR",
    "SHORES": "SHRS",
    "SKYWAY": "SKWY",
    "SPNG": "SPG",
    "SPNGS": "SPGS",
    "SPRING": "SPG",
    "SPRINGS": "SPGS",
    "SPRNG": "SPG",
    "SPRNGS": "SPGS",
    "SPURS": "SPUR",
    "SQR": "SQ",
    "SQRE": "SQ",
    "SQRS": "SQS",
    "SQU": "SQ",
    "SQUARE": "SQ",
    "SQUARES": "SQS",
    "STATION": "STA",
    "STATN": "STA",
    "STN": "STA",
    "STR": "ST",
    "STRAV": "STRA",
    "STRAVEN": "STRA",
    "STRAVENUE": "STRA",
    "STRAVN": "STRA",
    "STREAM": "STRM",
    "STREET": "ST",
    "STREETS": "STS",
    "STREME": "STRM",
    "STRT": "ST",
    "STRVN": "STRA",
    "STRVNUE": "STRA",
    "SUMIT": "SMT",
    "SUMITT": "SMT",
    "SUMMIT": "SMT",
    "TERR": "TER",
    "TERRACE": "TER",
    "THROUGHWAY": "TRWY",
    "TRACE": "TRCE",
    "TRACES": "TRCE",
    "TRACK": "TRAK",
    "TRACKS": "TRAK",
    "TRAFFICWAY": "TRFY",
    "TRAIL": "TRL",
    "TRAILER": "TRLR",
    "TRAILS": "TRL",
    "TRK": "TRAK",
    "TRKS": "TRAK",
    "TRLRS": "TRLR",
    "TRLS": "TRL",
    "TRNPK": "TPKE",
    "TUNEL": "TUNL",
    "TUNLS": "TUNL",
    "TUNNEL": "TUNL",
    "TUNNELS": "TUNL",
    "TUNNL": "TUNL",
    "TURNPIKE": "TPKE",
    "TURNPK": "TPKE",
    "UNDERPASS": "UPAS",
    "UNION": "UN",
    "UNIONS": "UNS",
    "VALLEY": "VLY",
    "VALLEYS": "VLYS",
    "VALLY": "VLY",
    "VDCT": "VIA",
    "VIADCT": "VIA",
    "VIADUCT": "VIA",
    "VIEW": "VW",
    "VIEWS": "VWS",
    "VILL": "VLG",
    "VILLAG": "VLG",
    "VILLAGE": "VLG",
    "VILLAGES": "VLGS",
    "VILLE": "VL",
    "VILLG": "VLG",
    "VILLIAGE": "VLG",
    "VIST": "VIS",
    "VISTA": "VIS",
    "VLLY": "VLY",
    "VST": "VIS",
    "VSTA": "VIS",
    "WALKS": "WALK",
    "WELL": "WL",
    "WELLS": "WLS",
    "WY": "WAY",
    "APARTMENT": "APT",
    "APPARTMENT": "APT",
    "BUILDING": "BLDG",
    "FLOOR": "FL",
    "SUITE": "STE",
    "ROOM": "RM",
    "DEPARTMENT": "DEPT",
};
