var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as genericObject from './baseSFObjClass.js';
import * as sfMethods from '../salesforce.js';
import * as sfUtilities from '../salesforceUtilities.js';
import { performAPIReq, endPointQueries } from '../salesforceAPI.js';
import * as objConstants from './constants/account.constants.js';
export { objConstants };
import * as locationObj from './location.js';
import * as subObj from './subscription.js';
import { sfApiNameInvoiceNumber } from './constants/subscription.constants.js';
class Account extends genericObject.generalObject {
    constructor() {
        super(objConstants.dataStructure, objConstants.apiMapping);
        this.details = this.createNiceObj({}, objConstants.dataStructure, objConstants.apiMapping);
        this.type = 'account';
        this.writeable = true;
        this.idReferenceName = "accountId";
    }
    getLocations(filters, roughMatch = false) {
        return __awaiter(this, void 0, void 0, function* () {
            filters = this.validateGetChild(filters);
            if (roughMatch) {
                filters = sfUtilities.setDupeFilters(filters);
            }
            return yield sfMethods.getBD2Locations(filters);
        });
    }
    getOrders(filters, orderStatus = null) {
        return __awaiter(this, void 0, void 0, function* () {
            filters = this.validateGetChild(filters);
            if (orderStatus) {
                return yield sfMethods.getBD2OrdersByStatus(orderStatus, filters);
            }
            else {
                return yield sfMethods.getBD2Orders(filters);
            }
        });
    }
    getInventory(filters = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            filters = this.validateGetChild(filters);
            return yield sfMethods.getBD2Inventories(filters);
        });
    }
    getReturns(filters = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            filters = this.validateGetChild(filters);
            //@ts-ignore
            delete filters.accountId; // << gets added by validateChildFunction
            let locations = yield sfMethods.getBD2Locations({ accountId: this.details.id });
            let returns = [];
            for (const location of locations) {
                returns = returns.concat(yield location.getReturns(filters));
            }
            return returns;
        });
    }
    getSubscriptions(filters = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            filters = this.validateGetChild(filters);
            return yield sfMethods.getBD2Subs(filters);
        });
    }
    createLocation() {
        return this.validateCreateChild(locationObj.createNew, "accountId");
    }
    createSubscription() {
        return this.validateCreateChild(subObj.createNew, "accountId");
    }
    getAllSubIds() {
        return __awaiter(this, void 0, void 0, function* () {
            let response = [];
            let query = sfUtilities.queryGenerator(subObj, { [subObj.objConstants.niceNameAccountId]: this.details.id }, [`${subObj.objConstants.niceNameInvoiceNumber}`]);
            let apiResp = yield performAPIReq("get", endPointQueries, 'url', query);
            if (apiResp.response && apiResp.response.records && apiResp.response.records.length > 0) {
                for (const respRow of apiResp.response.records) {
                    response.push(respRow[sfApiNameInvoiceNumber]);
                }
            }
            return response;
        });
    }
}
export function createNew() {
    return new Account();
}
