var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as genericObject from './baseSFObjClass.js';
import * as SalesForcer from '../salesforce.js';
import * as objConstants from './constants/order.constants.js';
import { exceptionalError } from '../salesforce.js';
export { objConstants };
class Order extends genericObject.generalObject {
    constructor() {
        super(objConstants.dataStructure, objConstants.apiMapping);
        this.type = 'order';
        this.writeable = true;
        this.details = this.createNiceObj({}, objConstants.dataStructure, objConstants.apiMapping);
    }
    createSF(update = false) {
        if (this.details.hasOwnProperty("money") && this.details.money.moneyTotal && this.details.money.moneyTotal > 0 && !this.details.money.moneyCurrency) {
            throw new exceptionalError("Can not create orders with money values and no currency");
        }
        let returnSF = super.createSF(update);
        return returnSF;
    }
    pauseOrder() {
        return __awaiter(this, void 0, void 0, function* () {
            this.validateBeforeUpdate([objConstants.orderStatusOrdered]);
            this.details.fulfillment.orderStatus = objConstants.orderStatusPaused;
            return yield this.updateRecord();
        });
    }
    resumeOrder() {
        return __awaiter(this, void 0, void 0, function* () {
            this.validateBeforeUpdate([objConstants.orderStatusPaused]);
            this.details.fulfillment.orderStatus = objConstants.orderStatusOrdered;
            return yield this.updateRecord();
        });
    }
    markFulfilled() {
        return __awaiter(this, void 0, void 0, function* () {
            this.validateBeforeUpdate([objConstants.orderStatusPaused, objConstants.orderStatusOrdered]);
            this.details.fulfillment.orderStatus = objConstants.orderStatusFulfilling;
            return yield this.updateRecord();
        });
    }
    cancelOrder() {
        return __awaiter(this, void 0, void 0, function* () {
            this.validateBeforeUpdate([objConstants.orderStatusPaused, objConstants.orderStatusOrdered]);
            this.details.fulfillment.orderStatus = objConstants.orderStatusCancelled;
            return yield this.updateRecord();
        });
    }
    addTracking(trackingDeets) {
        return __awaiter(this, void 0, void 0, function* () {
            const ogComment = `${this.details.notes}`;
            if (!Array.isArray(trackingDeets)) {
                throw new exceptionalError(`trackignDeets must be an array`);
            }
            let qtyRequested = this.details.fulfillment.qtyRequested;
            for (const trackingEntry of trackingDeets) {
                if (this.details.fulfillment.shopifyOrderID) {
                    this.details.fulfillment.shopifyOrderID = String(this.details.fulfillment.shopifyOrderID);
                }
                this.validateBeforeUpdate([objConstants.orderStatusPaused, objConstants.orderStatusOrdered, objConstants.orderStatusPartiallyShipped, objConstants.orderStatusFulfilling]);
                if (!trackingEntry.carrier || !trackingEntry.trackingNumber || trackingEntry.carrier === "" || trackingEntry.trackingNumber === "") {
                    throw new exceptionalError("Tracking carrier and number required");
                }
                if (ogComment.match(trackingEntry.trackingNumber)) {
                    continue;
                }
                if (!this.details.fulfillment.shippingTrackingCarrier) {
                    this.details.fulfillment.shippingTrackingCarrier = trackingEntry.carrier;
                }
                if (this.details.fulfillment.shippingTrackingCarrier !== trackingEntry.carrier) {
                    throw new exceptionalError("Currently all shipments must use same carrier");
                }
                if (!this.details.fulfillment.qtyFulfilled) {
                    this.details.fulfillment.qtyFulfilled = 0;
                }
                if (this.details.fulfillment.qtyFulfilled + trackingEntry.qty > qtyRequested) {
                    throw new exceptionalError(`Attempted to ship ${this.details.fulfillment.qtyFulfilled + trackingEntry.qty} units when requested is only ${qtyRequested}`);
                }
                this.details.fulfillment.qtyFulfilled += trackingEntry.qty;
                this.updateNotes(`Added tracking for ${trackingEntry.carrier} ${trackingEntry.trackingNumber} - Qty: ${trackingEntry.qty}`);
                if (this.details.fulfillment.qtyFulfilled === qtyRequested) {
                    this.details.fulfillment.orderStatus = objConstants.orderStatusShipped;
                }
                else if (this.details.fulfillment.orderStatus !== objConstants.orderStatusPartiallyShipped) {
                    this.details.fulfillment.orderStatus = objConstants.orderStatusPartiallyShipped;
                }
                if (!this.details.fulfillment.shippingTrackingNumber || this.details.fulfillment.shippingTrackingNumber === "") {
                    this.details.fulfillment.shippingTrackingNumber = `${trackingEntry.trackingNumber}`;
                }
                else {
                    this.details.fulfillment.shippingTrackingNumber = `${this.details.fulfillment.shippingTrackingNumber}\n${trackingEntry.trackingNumber}`;
                }
            }
            return yield this.updateRecord(ogComment === this.details.notes);
        });
    }
    markShipped(qty) {
        return __awaiter(this, void 0, void 0, function* () {
            this.validateBeforeUpdate([objConstants.orderStatusPaused, objConstants.orderStatusOrdered, objConstants.orderStatusPartiallyShipped, objConstants.orderStatusFulfilling]);
            if (!qty || typeof (qty) !== "number") {
                throw new exceptionalError("Final shipment qty is a required field");
            }
            if (qty > this.details.fulfillment.qtyRequested) {
                throw new exceptionalError(`Attempted to mark shipped with ${qty} units when requested is only ${this.details.fulfillment.qtyRequested}`);
            }
            this.updateNotes(`Forced status to shipped with manual qty of ${qty}`);
            this.details.fulfillment.qtyFulfilled = qty;
            return this.updateRecord(true);
        });
    }
    updateNotes(toAdd) {
        let now = `${(new Date()).toISOString()} | `;
        if (!this.details.notes) {
            this.details.notes = `${now}${toAdd}`;
        }
        else {
            this.details.notes = `${this.details.notes}\n${now}${toAdd}`;
        }
    }
    updateRecord(noComment = false) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!noComment) {
                let comment = `Order set to ${this.details.fulfillment.orderStatus}`;
                this.updateNotes(comment);
            }
            let update = yield SalesForcer.writeFile(this);
            this.details = update.details;
            return true;
        });
    }
    validateBeforeUpdate(validFields) {
        if (!this.loadedFromSF) {
            throw new exceptionalError("Only orders retrieved/created from SF may use this method");
        }
        if (validFields && !this.details.fulfillment.orderStatus || !validFields.includes(this.details.fulfillment.orderStatus)) {
            throw new exceptionalError(`Method is only valid for orders in state: ${validFields.join(" | ")}`);
        }
    }
}
export function createNew() {
    return new Order();
}
