var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import util from 'util';
import chai from 'chai';
import chaiAsPromised from "chai-as-promised";
import fetchMock from 'fetch-mock';
import * as testAccountCases from './testDetails/salesforce.cases.accounts.js';
import * as testAuthCases from './testDetails/salesforce.cases.auth.js';
import * as testLocationCases from './testDetails/salesforce.cases.locations.js';
import * as SalesForcer from '../salesforce.js';
import * as accountObj from '../sfObjects/account.js';
import * as SfAPI from '../salesforceAPI.js';
const expect = chai.expect;
chai.use(chaiAsPromised);
describe('Test the SalesForcer Account Level Functions', () => __awaiter(void 0, void 0, void 0, function* () {
    before(() => __awaiter(void 0, void 0, void 0, function* () {
        SfAPI.disconnect();
        fetchMock.restore();
        let authMock = testAuthCases.apiMockCases.goodTokenReq;
        fetchMock.post(authMock.req.url, { status: authMock.req.code, body: authMock.req.body });
        yield SfAPI.connect();
        fetchMock.restore();
    }));
    context('Create Accounts', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.post(/BD2_Account__c\/$/i, (url, opts) => __awaiter(void 0, void 0, void 0, function* () {
                const expectedAcctBody = JSON.stringify({
                    "Name": "A Test Account",
                    "User_Type__c": "BaaS Pro Shop",
                    "BD_User_ID__c": "123456465897",
                    "Contact_Email__c": "asdasd@asdasd.com",
                    "Contact_Name__c": "Chris Nevin",
                    "Contact_Phone__c": null,
                    "Accounting_Contact_Email__c": null,
                    "Accounting_Contact_Name__c": null,
                    "Accounting_Contact_Phone__c": null,
                    "Shipping_Email__c": null,
                    "CC_Emails__c": null,
                    "Bundle_SKUs__c": null,
                    "SKUs_to_Bundle__c": null,
                    "Shopify_Site__c": "Some Fake Test Store"
                });
                if (opts.body !== '') {
                    console.log(util.inspect(opts.body, { depth: null }));
                }
                return { body: JSON.stringify({
                        "id": "newAccountId",
                        "success": true,
                        "errors": []
                    }), status: 201 };
            }));
        }));
        it('Works', () => __awaiter(void 0, void 0, void 0, function* () {
            let newAcctDeets = {
                name: "Chris Nevin",
                email: "asdasd@asdasd.com",
                accountName: "A Test Account",
                phone: null,
                shopInstanceName: "Some Fake Test Store",
                customerId: "123456465897"
            };
            let expectedAcctObj = accountObj.createNew();
            expectedAcctObj.details.id = "newAccountId";
            expectedAcctObj.details.accountName = "A Test Account";
            expectedAcctObj.details.mainContact.contactName = "Chris Nevin";
            expectedAcctObj.details.mainContact.contactEmail = "asdasd@asdasd.com";
            expectedAcctObj.details.options.shopifyInstance = "Some Fake Test Store",
                //@ts-ignore
                expectedAcctObj.details.options.bulkFallbackPermitted = null;
            expectedAcctObj.details.servicesUserId = "123456465897";
            expectedAcctObj.details.type = "BaaS Pro Shop";
            let newAcct = yield SalesForcer.createAccount(newAcctDeets);
            console.log(util.inspect(newAcct.details, { depth: null }));
            expect(newAcct.details).to.deep.equal(expectedAcctObj.details);
        }));
    }));
    context('Test account search functions', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.get(testAccountCases.getAccountNoMatch.req.url, testAccountCases.getAccountNoMatch.req.body);
            fetchMock.get(testAccountCases.accountByNameOrIDWithLocations.req.url, testAccountCases.accountByNameOrIDWithLocations.req.body);
            fetchMock.get(testAccountCases.accountWithoutLocations.req.url, testAccountCases.accountWithoutLocations.req.body);
            fetchMock.get(testLocationCases.locationThatExists.req.url, testLocationCases.locationThatExists.req.body);
        }));
        it('Throws errors for invalid account fields', () => __awaiter(void 0, void 0, void 0, function* () {
            //@ts-ignore
            yield expect(SalesForcer.getBD2Accounts({ yert: "lol" })).to.be.rejectedWith('Unsupported filter yert for query type BD2_Account__c');
        }));
        it('Handles the no-match state by returning null', () => __awaiter(void 0, void 0, void 0, function* () {
            let noMatch = yield SalesForcer.getBD2Accounts({ accountName: "Bob Of Mainframe" });
            expect(noMatch).to.be.empty;
        }));
        it('Gets account by name', () => __awaiter(void 0, void 0, void 0, function* () {
            let accountByName = yield SalesForcer.getBD2Accounts({ accountName: "ACME" });
            expect(accountByName[0].details).to.deep.equal(testAccountCases.accountByNameOrIDWithLocations.resp);
        }));
        it('Get account by ID', () => __awaiter(void 0, void 0, void 0, function* () {
            let accountByName = yield SalesForcer.getBD2Accounts({ id: "someRandomIdLookATthis0mg" });
            expect(accountByName[0].details).to.deep.equal(testAccountCases.accountByNameOrIDWithLocations.resp);
        }));
    }));
}));
