var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import chai from 'chai';
import chaiAsPromised from "chai-as-promised";
import fetchMock from 'fetch-mock';
import * as testAuthCases from './testDetails/salesforce.cases.auth.js';
import * as testReturnCases from './testDetails/salesforce.cases.returns.js';
import * as locationObj from '../sfObjects/location.js';
import * as SalesForcer from '../salesforce.js';
import * as SfAPI from '../salesforceAPI.js';
const expect = chai.expect;
chai.use(chaiAsPromised);
describe('Test the SalesForcer module', () => __awaiter(void 0, void 0, void 0, function* () {
    before(() => __awaiter(void 0, void 0, void 0, function* () {
        SfAPI.disconnect();
        fetchMock.restore();
        let authMock = testAuthCases.apiMockCases.goodTokenReq;
        fetchMock.post(authMock.req.url, { status: authMock.req.code, body: authMock.req.body });
        yield SfAPI.connect();
        fetchMock.restore();
    }));
    context('Test return search by id', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.get(testReturnCases.returnById.req.url, testReturnCases.returnById.req.body);
            fetchMock.get(testReturnCases.returnByName.req.url, testReturnCases.returnByName.req.body);
            fetchMock.get(testReturnCases.dontExist.req.url, testReturnCases.dontExist.req.body);
            fetchMock.get(testReturnCases.notUnique.req.url, testReturnCases.notUnique.req.body);
        }));
        it('Validates inputs', () => __awaiter(void 0, void 0, void 0, function* () {
            //@ts-ignore
            yield expect(SalesForcer.getBD2Returns({ asdf: "asdadasd" })).to.be.rejectedWith('Unsupported filter asdf for query type BD2_Return__c');
        }));
        it('Returns null when not found', () => __awaiter(void 0, void 0, void 0, function* () {
            const returnByRefNum = yield SalesForcer.getBD2Returns({ referenceNumber: "dontexist" });
            expect(returnByRefNum).to.be.empty;
        }));
        it('Returns a return by name', () => __awaiter(void 0, void 0, void 0, function* () {
            const returnByName = yield SalesForcer.getBD2Returns({ recordName: "BD2Ret-000069" });
            expect(returnByName[0].details).to.deep.equal(testReturnCases.returnByName.resp);
        }));
        it('Returns a return by id', () => __awaiter(void 0, void 0, void 0, function* () {
            const returnById = yield SalesForcer.getBD2Returns({ id: "someRandomId" });
            expect(returnById[0].details).to.deep.equal(testReturnCases.returnById.resp);
        }));
        it('Returns a return by reference number', () => __awaiter(void 0, void 0, void 0, function* () {
            const returnByRefNum = yield SalesForcer.getBD2Returns({ referenceNumber: "someRandomId" });
            expect(returnByRefNum[0].details).to.deep.equal(testReturnCases.returnById.resp);
        }));
    }));
    context('Test searching for returns by source + filters', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.get(testReturnCases.returnById.req.url, testReturnCases.returnById.req.body);
        }));
        it('Returns returns by location name - exists but no returns', () => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.get(testReturnCases.dontExist.req.url, testReturnCases.dontExist.req.body);
            fetchMock.get(testReturnCases.locationsBelongingToAccountNoReturns.req.url, testReturnCases.locationsBelongingToAccountNoReturns.req.body);
            const location = yield SalesForcer.getBD2Locations({ locationName: "ARealPlaceThatExists" });
            const returns = yield location[0].getReturns();
            expect(returns).to.be.empty;
        }));
        it('Returns returns by account - exists but has no locations', () => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.get(testReturnCases.accountNoLocations.req.url, testReturnCases.accountNoLocations.req.body);
            fetchMock.get(testReturnCases.locationDoesntExist.req.url, testReturnCases.locationDoesntExist.req.body);
            const account = yield SalesForcer.getBD2Accounts({ accountName: "iHaveNoLocations" });
            const returns = yield account[0].getReturns();
            expect(returns).to.be.empty;
        }));
        it('Returns returns by location by name that were faulty', () => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.get(testReturnCases.locationWithOneReturn.req.url, testReturnCases.locationWithOneReturn.req.body);
            fetchMock.get(testReturnCases.returnByLocationFaulty.req.url, testReturnCases.returnByLocationFaulty.req.body);
            const location = yield SalesForcer.getBD2Locations({ id: "locationWithReturnsId1" });
            const returns = yield location[0].getReturns({ testResults: "Failed" });
            expect(returns.map((a) => (a.details))).to.deep.equal(testReturnCases.returnByLocationFaulty.resp);
        }));
        it('Returns returns by account by id that have been returned to inventory RETURN', () => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.get(testReturnCases.accountWithLocationsThatHaveReturns.req.url, testReturnCases.accountWithLocationsThatHaveReturns.req.body);
            fetchMock.get(testReturnCases.locationsWithReturns.req.url, testReturnCases.locationsWithReturns.req.body);
            fetchMock.get(testReturnCases.returnByAccountOne.req.url, testReturnCases.returnByAccountOne.req.body);
            fetchMock.get(testReturnCases.returnByAccountTwo.req.url, testReturnCases.returnByAccountTwo.req.body);
            const account = yield SalesForcer.getBD2Accounts({ id: "someIdWithKids" });
            const returns = yield account[0].getReturns({ fate: "Returned to Inventory", inventoryId: "RETURN" });
            expect(returns.map((a) => (a.details))).to.deep.equal(testReturnCases.returnByAccountOne.resp);
        }));
    }));
    context('Creates returns for locations', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.post(/BD2_Return__c\/$/i, (url, opts) => __awaiter(void 0, void 0, void 0, function* () {
                return testReturnCases.checkReturnBody(url, opts);
            }));
        }));
        it('Does usual validations', () => __awaiter(void 0, void 0, void 0, function* () {
            let parentLocation = locationObj.createNew();
            parentLocation.loadedFromSF = true;
            parentLocation.details.id = "okIcreate";
            let newReturn = parentLocation.createReturn();
            yield expect(SalesForcer.writeFile(newReturn)).to.be.rejectedWith('Missing mandatory value topLevel -> testResults from input');
        }));
        it('Creates a return by location', () => __awaiter(void 0, void 0, void 0, function* () {
            let parentLocation = locationObj.createNew();
            parentLocation.loadedFromSF = true;
            parentLocation.details.id = "okIcreate";
            let newReturn = parentLocation.createReturn();
            newReturn.details.testResults = "Untested";
            newReturn = (yield SalesForcer.writeFile(newReturn));
            expect(newReturn.details.id).to.equal('aLocationWithReturns!');
        }));
    }));
    context('Updates returns', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.post(/BD2_Return__c\/$/i, (url, opts) => __awaiter(void 0, void 0, void 0, function* () {
                return testReturnCases.checkReturnBody(url, opts);
            }));
            fetchMock.patch(/BD2_Return__c\/\w+?/i, (url, opts) => __awaiter(void 0, void 0, void 0, function* () {
                return testReturnCases.checkReturnBody(url, opts);
            }));
        }));
        it('Updates a return to set passed and returned and assigned to inventory', () => __awaiter(void 0, void 0, void 0, function* () {
            let parentLocation = locationObj.createNew();
            parentLocation.loadedFromSF = true;
            parentLocation.details.id = "okIcreate";
            let newReturn = parentLocation.createReturn();
            newReturn.details.testResults = "Untested";
            newReturn = (yield SalesForcer.writeFile(newReturn));
            newReturn.details.inventoryId = "someMasterInventory";
            newReturn.details.testResults = "Passed";
            newReturn.details.fate = "Returned to Inventory";
            yield expect(SalesForcer.writeFile(newReturn)).not.to.be.rejected;
        }));
    }));
}));
